/*
 * GameEngine.java
 * 
 * Copyright (c) 2004 ICEGT, Georgia Institute of Technology, Jeremy Goecks
 */
package edu.gatech.icegt.guesswho;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Random;
import java.util.Vector;

/**
 * Class manages a GuessWhoGame game, managing the game's logic and enabling clients
 * to query or update the game's state.
 * 
 * @author Jeremy Goecks
 * @version 0.1
 */
public class GameEngine {
	
	/** All possible people that can be in a game. */
	protected Person[] allPeople;
	
	/** People in the current game. */
	protected Person[] peopleInCurGame;
	
	/** Person who is the answer to the current game. */
	protected Person answer;
	
	/** Create game. */
	public GameEngine(Person[] allPeople) {
		this.allPeople = allPeople;
	} // end GameLogic()
	
	/** Get people in the current game. */
	public Person[] getPeopleInCurGame() {
		return peopleInCurGame;
	} // end getPeopleInCurGame()
	
	/** 
	 * Start a new game. Method randomly chooses a people for the game and then 
	 * chooses a person to be the answer to the game.
	 * @param numPeopleInGame number of people in game; if number exceeds the 
	 * number of possible people, all people are used in the game.
	 */
	public void startNewGame(int numPeopleInGame) {
		// Random number generator.
		Random rng = new Random();
		
		// Get people for game.
		if (numPeopleInGame < allPeople.length) {
			// Get new people for game.
			Hashtable peopleInGameTable = new Hashtable();
			Person person;
			int index;
			while (peopleInGameTable.size() < numPeopleInGame) {
				// Get index, person.
				index = (int)Math.floor(rng.nextDouble()*allPeople.length);
				person = allPeople[index];
				
				// Add person to table if he/she is not already in table.
				if (peopleInGameTable.get(person) == null)
					peopleInGameTable.put(person, person);
			} // end while
			
			// Copy people from table to [].
			peopleInCurGame = new Person[numPeopleInGame];
			index = 0;
			for (Enumeration enum = peopleInGameTable.keys(); 
					enum.hasMoreElements() ;)
				peopleInCurGame[index++] = (Person)enum.nextElement();
		} // end if
		else { // numPeopleInGame >= allPeople.length
			// Use all people.
			peopleInCurGame = allPeople;
		} // end else
		
		// Set random person to be the answer.
		int randomIndex = (int)Math.floor(rng.nextDouble()*peopleInCurGame.length);
		answer = peopleInCurGame[randomIndex];
	} // end startNewGame()
	
	/**
	 * Get the fun fact for the person that is the answer.
	 */
	public String getAnswersFunFact() {
		// If answer is null, return ""
		return (answer != null ? answer.getFunFact() : "");
	} // end getFunFactForAnswer()
	
	/**
	 * Returns true if person is the answer, false otherwise.
	 */
	public boolean isAnswer(Person p) {
		return answer.equals(p);
	} // end isAnswer()
	
	/**
	 * Returns true if answer has a given gender.
	 * @param gender
	 */
	public boolean isAnswersGender(String gender) {
		return (answer.getGender().equals(gender));
	} // end isAnswersGender()
	
	/**
	 * Returns true if answer has a given hair color.
	 * @param hairColor
	 */
	public boolean isAnswersHairColor(String hairColor) {
		return (answer.getHairColor().equals(hairColor));
	} // end isAnswersHairColor()
	
	/**
	 * Returns true if answer has a given eye color.
	 * @param eyeColor
	 */
	public boolean isAnswersEyeColor(String eyeColor) {
		return (answer.getEyeColor().equals(eyeColor));
	} // end isAnswersEyeColor()
	
	/**
	 * Returns true if answer's earring status matches the given status.
	 * @param answerHasEarrings
	 */
	public boolean isAnswersHasEarrings(boolean hasEarrings) {
		return (answer.hasEarrings() == hasEarrings);
	} // end isAnswersHasEarrings()
	
	/**
	 * Method returns the people in the game that fit a set of criteria. If 
	 * an attribute is null, that attribute is not considered to be a criterion.
	 * @param gender gender criterion
	 * @param hairColor a hair color
	 * @param eyeColor an eye color
	 * @param answerHasEarrings has earrings criterion
	 */
	public Person[] getPeopleThatFitCriteria(String gender, String hairColor, 
											String eyeColor, Boolean hasEarrings) {
		//
		// Get people in game that fit criteria.
		//
		Vector peopleThatFitCriteria = new Vector();
		Person p;
		boolean fitsCriteria;
		for (int i = 0; i < peopleInCurGame.length; i++) {
			// Get person.
			p = peopleInCurGame[i];
			
			// Determine if person fits criteria.
			fitsCriteria = true;
			
			// Is person the given gender?
			if ( (gender != null) && (!p.getGender().equals(gender)) )
				fitsCriteria = false;
			
			// Does person have the given hair color?
			if ( (hairColor != null) && (!p.getHairColor().equals(hairColor)) )
				fitsCriteria = false;

			// Does person have the given eye color?
			if ( (eyeColor != null) && (!p.getEyeColor().equals(eyeColor)) )
				fitsCriteria = false;
			
			// Does person match hasEarrings?
			if ( (hasEarrings != null) && 
				 (p.hasEarrings != hasEarrings.booleanValue()) )
				fitsCriteria = false;
			
			// If person matches criteria, add person to list.
			if (fitsCriteria)
				peopleThatFitCriteria.addElement(p);
		} // end for
		
		// Convert Vector to [].
		Person[] returnVal = new Person[peopleThatFitCriteria.size()];
		peopleThatFitCriteria.copyInto(returnVal);
		
		return returnVal;
	} // end getPeopleThatFitCriteria()
	
	/**
	 * Method returns the people in the game that fit a set of criteria. If 
	 * an attribute is null, that attribute is not considered to be a criterion.
	 * @param gender gender criterion
	 * @param hairColors a list of hair colors
	 * @param eyeColors a list of eye colors
	 */
	public Person[] getPeopleThatFitCriteria(String gender, String hairColors[], 
												String[] eyeColors) {
		//
		// Get people in game that fit criteria.
		//
		Vector peopleThatFitCriteria = new Vector();
		Person p;
		boolean fitsCriteria;
		boolean inSet;
		String hairColor, eyeColor;
		for (int i = 0; i < peopleInCurGame.length; i++) {
			// Get person.
			p = peopleInCurGame[i];
			
			// Determine if person fits criteria.
			fitsCriteria = true;
			
			// Is person the given gender?
			if ( (gender != null) && (!p.getGender().equals(gender)) )
				fitsCriteria = false;
			
			// Does person have a given hair color?
			if (hairColors != null) {
				inSet = false;
				hairColor = p.getHairColor();
				for (int j = 0; j < hairColors.length; j++)
					if (hairColors[j].equals(hairColor)) {
						// Found person's hair color in set.
						inSet = true;
						break;
					} // end if
				
				// Person does _not_ fit criteria if hair color is not in set.
				if (!inSet)
					fitsCriteria = false;
			} // end if
			
			// Does person have a given eye color?
			if (eyeColors != null) {
				inSet = false;
				eyeColor = p.getEyeColor();
				for (int j = 0; j < eyeColors.length; j++)
					if (eyeColors[j].equals(eyeColor)) {
						// Found person's eye color in set.
						inSet = true;
						break;
					} // end if
				
				// Person does _not_ fit criteria if eye color is not in set.
				if (!inSet)
					fitsCriteria = false;
			} // end if
			
			
			// If person matches criteria, add person to list.
			if (fitsCriteria)
				peopleThatFitCriteria.addElement(p);
		} // end for
		
		// Convert Vector to [].
		Person[] returnVal = new Person[peopleThatFitCriteria.size()];
		peopleThatFitCriteria.copyInto(returnVal);
		
		return returnVal;
	} // end getPeopleThatFitCriteria()
	
	/**
	 * Method returns the people in the game that fit a set of criteria. If 
	 * an attribute is null, that attribute is not considered to be a criterion.
	 * @param gender gender criterion
	 * @param hairColors a list of hair colors
	 * @param eyeColors a list of eye colors
	 * @param answerHasEarrings has earrings criterion
	 */
	public Person[] getPeopleThatFitCriteria(String gender, String hairColors[], 
												String[] eyeColors, boolean hasEarrings) {
		//
		// Get people in game that fit criteria.
		//
		Vector peopleThatFitCriteria = new Vector();
		Person p;
		boolean fitsCriteria;
		boolean inSet;
		String hairColor, eyeColor;
		for (int i = 0; i < peopleInCurGame.length; i++) {
			// Get person.
			p = peopleInCurGame[i];
			
			// Determine if person fits criteria.
			fitsCriteria = true;
			
			// Is person the given gender?
			if ( (gender != null) && (!p.getGender().equals(gender)) )
				fitsCriteria = false;
			
			// Does person have a given hair color?
			if (hairColors != null) {
				inSet = false;
				hairColor = p.getHairColor();
				for (int j = 0; j < hairColors.length; j++)
					if (hairColors[j].equals(hairColor)) {
						// Found person's hair color in set.
						inSet = true;
						break;
					} // end if
				
				// Person does _not_ fit criteria if hair color is not in set.
				if (!inSet)
					fitsCriteria = false;
			} // end if
			
			// Does person have a given eye color?
			if (eyeColors != null) {
				inSet = false;
				eyeColor = p.getEyeColor();
				for (int j = 0; j < eyeColors.length; j++)
					if (eyeColors[j].equals(eyeColor)) {
						// Found person's eye color in set.
						inSet = true;
						break;
					} // end if
				
				// Person does _not_ fit criteria if eye color is not in set.
				if (!inSet)
					fitsCriteria = false;
			} // end if
			
			// Does person match hasEarrings?
			if ((p.hasEarrings != hasEarrings) )
				fitsCriteria = false;
			
			// If person matches criteria, add person to list.
			if (fitsCriteria)
				peopleThatFitCriteria.addElement(p);
		} // end for
		
		// Convert Vector to [].
		Person[] returnVal = new Person[peopleThatFitCriteria.size()];
		peopleThatFitCriteria.copyInto(returnVal);
		
		return returnVal;
	} // end getPeopleThatFitCriteria()
	
} // end class GameLogic

